#copyright ReportLab Europe Limited. 2000-2012
#see license.txt for license details
__version__=''' $Id$ '''
__doc__="""Script stub for pdfencryption.

This can be shipped in plain mode outside of any encrypted archives.
"""
from reportlab.lib.pdfencrypt import *

if __name__ == '__main__':
    main()



#copyright ReportLab Europe Limited. 2000-2012
from os.path import splitext, isfile, abspath, isdir, normpath, basename
from fnmatch import fnmatch
import os, sys, time, string
try:
    from hashlib import md5
except ImportError:
    from md5 import md5

class FilenameClassifier:
    '''provides simple text/binary filename classification services'''
    txt_exts = [".afm", ".py", ".prep", ".rml", ".xml", ".html", ".txt", ".dtd", ".h", ".c", ".iss", '.js', '.css', '.pas']
    txt_filenames = ['readme*','changes*']

    def getTextExtensions(self):
        "file extensions for this pp which should be treated as ascii"
        return self.txt_exts

    def hasTextFileName(self,filename):
        fn = basename(filename)
        for pat in self.txt_filenames:
            if fnmatch(fn,pat): return 1
        return 0

    def isTextFile(self, filename):
        return (self.hasTextFileName(filename) or splitext(filename)[1] in self.getTextExtensions())

    def isBinaryFile(self, filename):
        return not self.isTextFile(filename)

    def mode(self,filename):
        '''return mode character b or t'''
        return ('b','t')[self.isTextFile(filename)]

class Manifester(FilenameClassifier):
    '''Mix in class providing some standard methods for dealing with manifests'''

    def __init__(self,dirname=None,id='',verbose=0):
        self.id = id
        self.verbose = verbose
        if dirname:
            assert isdir(dirname), "Directory %s not found!" % dirname
        else:
            dirname = os.getcwd()
        self.baseDir = normpath(abspath(dirname))
        if self.verbose: print('running in directory %s' % self.baseDir)
        self.resetDir()

    def resetDir(self):
        "Returns current directory to my base"
        os.chdir(self.baseDir)

    def getMachineInfo(self):
        "Something to identify machine"
        import socket
        return socket.gethostname()

    if sys.hexversion >= 0x02000000:
        def _digester(self,s):
            return md5(s).hexdigest()
    else:
            # hexdigest not available in 1.5
        def _digester(self,s):
            return ''.join(["%02x" % ord(x) for x in md5(s).digest()])

    def _clean(self,data):
        data = data.replace('\r','')
        data = data.replace('\n','')
        return data

    def getTextFileChecksum(self, filename, _id=0):
        "checksum which is robust against line ending problems"
        txt = self._clean(open(filename).read())
        r = self._digester(txt)
        if _id:
            L = string.split(txt)
            for h in  ('#$'+'Header:', '$'+'Header:', '$'+'Id:', '#$'+'Id:', '/*$'+'Id:', '/*$'+'Header:'):
                try:
                    i = L.index(h)
                    return r + ('\t# %s %s %s' % (L[i+2],L[i+3],L[i+4]))
                except (IndexError, ValueError):
                    pass
        return r

    def getBinaryFileChecksum(self, filename):
        "ordinary checksum for all file contents"
        return self._digester(open(filename, 'rb').read())

    def getFileChecksum(self, filename, _id=0):
        "returns unique fingerprint for this file"
        if self.isTextFile(filename):
            return self.getTextFileChecksum(filename,_id=_id)
        else:
            return self.getBinaryFileChecksum(filename)

    def writeSignedManifest(self,fList,mName,_id=0,verbose=None):
        if verbose is None: verbose = getattr(self,'verbose')
        self.resetDir()
        L = ['#signed %s for %s in %s generated on %s at %s' % (mName, self.id, self.baseDir,
                            self.getMachineInfo(), time.ctime(time.time()))]
        M = []
        for f in fList:
            if not isfile(f):
                M.append(f)
                if verbose>1: print('not computing checksum for missing', f)
            else:
                if verbose>1: print('computing checksum for', f)
                L.append('%s\t%s' % (f, self.getFileChecksum(f,_id=_id)))
        L.append('%s\t%s' % (mName, self._digester(string.join(L,''))))
        open(mName,'w').write(string.join(L,'\n'))
        if verbose: print('wrote '+mName)
        assert not M, "Missing files %s" % M

    def stripComments(self, list_of_strings):
        "Removes blank lines and anything after a hash comment"
        entries = []
        for line in list_of_strings:
            beforeComment = line.split('#')[0].strip()
            if beforeComment:
                entries.append(beforeComment)
        return entries

    def _getSignedManifest(self,mName):
        '''return a line ending normalised list of lines'''
        self.resetDir()
        try:
            L = string.replace(open(mName,'rb').read(),'\r\n','\n')
            L = [_f for _f in string.split(string.replace(L,'\r','\n'),'\n') if _f]
        except:
            L = None
        return L

    def verifySignedManifest(self,mName,verbose=None):
        """Reads and checks against a signed manifest.  Returns
        3-tuple of (pass/fail, failed files, missing files)"""
        if verbose is None: verbose = getattr(self,'verbose')
        L = self._getSignedManifest(mName)
        if L:
            L0 = L[0]
            n = len(L)
        else:
            n = 0
        failed = []
        missing = []
        if not n:
            if verbose: print((L is None and 'File %s missing or invalid'   or  'File %s empty') % mName)
        else:
            # check the bottom row, a checksum for the manifest itself
            l = L[-1]
            L = L[1:-1]
            if l:
                l = self.stripComments([l])[0]
                (f, csum) = l.split()
                ok = self._digester(string.join([L0]+L,''))==csum
            else:
                ok = 0
            L = self.stripComments(L)
            if not ok:
                failed = [f]
                if verbose: print('%s failed!\nAbandoning checks for other files.' % f)
                for l in L:
                    (f, csum) = l.split()
                    failed.append(f)
            else:
                for l in L:
                    (f, csum) = l.split()
                    if isfile(f):
                        if self.getFileChecksum(f)!=csum:
                            failed.append(f)
                            if verbose>1: print('%s failed' % f)
                    else:
                        missing.append(f)
                        if verbose > 1: print('%s missing' % f)

        if verbose: print('%d files verified, %d failed, %d missing' % (n - len(failed) - len(missing), len(failed), len(missing)))
        return ((n>0 and failed==[] and missing==[]), failed, missing)

if __name__=='__main__': #noruntest
    for fn in sys.argv[1:]:
        Manifester().verifySignedManifest(fn,verbose=1)



# this updates the template for all pages on the site.
import os
import sys

#page template
TEMPLATE_FN='../htdocs/reportlab_template.html'
START_DELIM = '<!--User Content Starts Here-->'
END_DELIM = '<!--User Content Ends Here-->'

DUAL_FMT_HTML='<title>%(title)s</title>'+START_DELIM+'''
<H1><center>%(title)s</center></H1>
<center>This page is also available in <a href="%(pdfname)s"><b>PDF</b></a></center>
<p>
%(body)s
<p>
<center>This page is also available in <a href="%(pdfname)s"><b>PDF</b></a>
'''+END_DELIM

DUMB_FMT_HTML='<title>%(title)s</title>'+START_DELIM+'''
%(heading)s
<p>
%(body)s
<p>
'''+END_DELIM

def _no_file_error(fn):
	raise "Can't find template file \"%s\""%fn
def	find_template(template_fn):
	if os.path.isabs(template_fn):
		if not os.path.isfile(template_fn): _no_file_error(template_fn)
		return template_fn

	pp = ''
	while 1:
		p  = os.getcwd()
		if p==pp: _no_file_error(template_fn)
		fn = os.path.join(p,template_fn)
		if os.path.isfile(fn): return fn
		pp = p
		os.chdir('..')

def DoNormalPages(pages):
	for filename in pages:
		if os.path.isfile(filename):
			applyTemplate(filename, find_template(TEMPLATE_FN))
		else:
			print('    Specified file %s not found' % filename)

def get_templated_HTML( src_template=DUMB_FMT_HTML, title=None, heading=None, body=None):
	from reportlab.lib.utils import getStringIO
	p = DocStyle0HTML()
	D = p.processfile( getStringIO(body), None )
	if title is not None: D['title'] = title
	if heading is not None: D['heading'] = '<center><h1>%s</h1></center>' % heading
	return get_templated_pagedata( find_template(TEMPLATE_FN), src_template % D)

def get_templated_raw_HTML( src_template=DUMB_FMT_HTML, template_fn=TEMPLATE_FN, title=None, heading=None, body=None):
	D = {}
	D['title'] = title
	D['heading'] = '<center><h1>%s</h1></center>' % heading
	D['body'] = body
	return get_templated_pagedata( find_template(template_fn), src_template % D)

def add2head(s,meta):
	return s.replace('<head>','<head>'+meta)

def DoDualFormatPages(pages):
	for filename in pages:
		if os.path.isfile(filename):
			p=DocStyle0PDF()
			pdfname = os.path.splitext(filename)[0]+'.pdf'
			p.processfile(filename, pdfname )
			p = DocStyle0HTML()
			D = p.processfile( filename, None )
			D['pdfname'] = pdfname
			p = DUAL_FMT_HTML % D
			_applyTemplate(p, os.path.splitext(filename)[0]+'.html',  find_template(TEMPLATE_FN))
		else:
			print('    Specified file %s not found' % filename)

def get_templated_pagedata( templatefile, rawdata):
	rawtemplate = open(templatefile, 'r').read()
	#dirty hacks for cgi scripts resources
	rawtemplate = rawtemplate.replace('rsrc/', '../rsrc/')
	rawtemplate = rawtemplate.replace('favicon.ico', '/favicon.ico')
	rawtemplate = rawtemplate.replace('../rsrc/rl_logo_16x16.gif', '/rsrc/rl_logo_16x16.gif')
	js='''	function mr(base, ar, start){
		var i, s;
		for(i=start;i<ar.length;i++){
			s = ar[i];
			if(!s) continue;
			if(s.length>3){
				mr(base, s, 3);	
			}
			else
				if((s[1].indexOf('/')!=0) && (s[1].indexOf('http:/')!=0))
					s[1] = base+s[1];
		}
	}
	mr('../', MENU_ITEMS, 0);'''
	rawtemplate = rawtemplate.replace('//<!-- MENUMODIFY -->', js)
	pagedata = intersperse(rawtemplate, rawdata, '<title>','</title>')
	return intersperse(pagedata, rawdata, START_DELIM, END_DELIM)

def _applyTemplate(rawdata, destfile, templatefile):
	'''
	As applyTemplate, but works on the rawdata not a file
	destfile gets the result.
	'''
	open(destfile, 'w').write(get_templated_pagedata(templatefile,rawdata))
	print('    File %s updated.' % destfile)

def applyTemplate(sourcefile, templatefile):
	"""recreates the source file using its content block and the template file.
	The important content of the source file is presumed to be between two
	blocks exactly like this (defined above):
		<!--User Content Starts Here-->
		<!--User Content Ends Here-->
	If they are not found, no action is taken.
	"""
	_applyTemplate(open(sourcefile, 'r').read(), sourcefile, templatefile)


def intersperse(data1, data2, delim1, delim2):
	"""takes data1 up to start of delim1; data2 between delim1 and delim2;
	and data1 from the end of delim2 onwards.  Used to grab the content
	block and the title out of source pages. Accepts any case in the delimiters.
	Returns the interspersed data, or the original (data1) is nothing found.

	Example:
		data1 = 'Made By ReportLab <TITLE>Title Goes Here</Title> 2000-02-28'
		data2 = 'Made by PageMill <TITLE>MyPage</TITLE> copyright Adobe'
	When interspersed, you get 'MyPage' in the first line.
	"""
	data1_delim1_start = data1.lower().find( delim1.lower())
	if data1_delim1_start == -1:
		return data1
	data1_delim2_start = data1.lower().find(delim2.lower())
	if data1_delim2_start == -1:
		return data1
	data2_delim1_start = data2.lower().find(delim1.lower())
	if data2_delim1_start == -1:
		return data1
	data2_delim2_start = data2.lower().find(delim2.lower())
	if data2_delim2_start == -1:
		return data1
	#if we get this far, the two strings match and can be interspersed
	combined = data1[0:data1_delim1_start] + \
			   data2[data2_delim1_start:data2_delim2_start] + \
			   data1[data1_delim2_start:]
	return combined

"""
simple document formatting (quickhack) for pdf:

.t title

.h Header

This is a paragraph.
This is in the same paragraph.

This is the start of the next paragraph.

.i Item title:
Item detail.
more detail.

.i next item title:
and so forth.

.u URL definition as URL (annotation text)

.H stuff for HTML ONLY you've been warned RGB :)
"""
import reportlab
from reportlab import platypus
inch = reportlab.lib.units.inch

do_images = 1

class DocStyle0PDF:
	Title = "Untitled Document"
	mode = "normal"

	def __init__(self):
		self.elts = []
		self.paragraph = []
		styles = reportlab.lib.styles.getSampleStyleSheet()
		self.paraStyle = styles["Normal"]
		self.headerStyle = styles["Heading3"]
		self.preStyle = styles["Code"]
		self.modeStyles = {"normal": self.paraStyle, "preformatted": self.preStyle}
		self.modeFormat = {"normal": platypus.paragraph.Paragraph, "preformatted": platypus.flowables.Preformatted}
		self.mode = "normal"

	def add_text(self, text):
		text1 = text.strip()
		if self.mode=="normal":
			text = text1
			if not text:
				self.end_paragraph()
			else:
				self.paragraph.append(text)
		elif self.mode=="preformatted":
			if text1:
				self.paragraph.append(text)
			else:
				self.end_paragraph()
		else:
			raise ValueError("invalid mode")

	def end_paragraph(self):
		p = self.paragraph
		if self.mode=="normal":
			if p:
				body = " ".join(p)
				self.emit_paragraph(body)
		elif self.mode=="preformatted":
			body = "".join(p)
			self.emit_paragraph(body)
		self.paragraph = []

	def emit_paragraph(self, body):
		style = self.modeStyles[self.mode]
		formatter = self.modeFormat[self.mode]
		t = formatter(body, style)
		s2 = platypus.flowables.Spacer(10, 10)
		e = self.elts
		e.append(t)
		e.append(s2)

	def addTitle(self, title1):
		self.Title = title1

	def addHeader(self, header):
		self.addItem(header)
		s2 = platypus.flowables.Spacer(0.2*inch, 0.2*inch)
		e = self.elts
		e.append(s2)

	def addItem(self, item):
		s1 = platypus.flowables.Spacer(0.2*inch, 0.2*inch)
		t = platypus.paragraph.Paragraph(item, self.headerStyle)
		e = self.elts
		e.append(s1)
		e.append(t)

	def addURL(self, URLdef):
		URLdef = URLdef.split()
		self.paragraph.append(' <'+URLdef[0]+'> ')

	def myFirstPage(self, canvas, doc):
		canvas.saveState()
		canvas.setFillColorRGB(0,0,0.8)
		canvas.rect(20,20,90,802,stroke=0, fill=1)
		#if do_images:
		#	canvas.drawInlineImage("images/replog.gif",20,750,90,60)
		#	canvas.drawInlineImage("images/reppow.jpg", 2*inch, platypus.PAGE_HEIGHT-3*inch)
		canvas.setStrokeColorRGB(1,0,0)
		canvas.setLineWidth(5)
		canvas.setFont("Helvetica-Bold",16)
		canvas.drawCentredString(4*inch, reportlab.lib.units.PAGE_HEIGHT-4*inch, self.Title)
		canvas.setFont('Times-Roman',9)
		canvas.drawString(2*inch, reportlab.lib.units.PAGE_HEIGHT-0.50 * inch, "First Page / %s" % self.Title)
		canvas.restoreState()

	def myLaterPages(self, canvas, doc):
		canvas.saveState()
		canvas.setFillColorRGB(0,0,0.8)
		canvas.rect(20,20,90,802,stroke=0, fill=1)
		#if do_images:
		#	canvas.drawInlineImage("images/replog.gif",20,750,90,60)
		canvas.setFont('Times-Roman',9)
		pageinfo=self.Title
		canvas.drawString(2*inch, reportlab.lib.units.PAGE_HEIGHT-0.5 * inch, "Page %d %s" % (doc.page, pageinfo))
		canvas.restoreState()

	def process(self, lines, tofilename):
		for l in lines:
			if l[0]==".":
				tag = l[1]
				remainder = l[2:]
				if tag=="t":
					self.addTitle(remainder)
				elif tag=="h":
					self.addHeader(remainder)
				elif tag=="i":
					self.addItem(remainder)
				elif tag=="u":
					self.addURL(remainder)
				elif tag=="p":
					self.mode = "preformatted"
				elif tag=='H' and self.__class__ is DocStyle0HTML:
					self.add_text(remainder)
				elif tag=="n":
					self.end_paragraph()
					self.mode = "normal"
			else:
				self.add_text(l)
		self.end_paragraph()
		if tofilename is None:
			return self.finish(tofilename)
		else:
			self.finish(tofilename)

	def finish(self, tofilename):
		doc = platypus.doctemplate.SimpleFlowDocument(tofilename, reportlab.lib.units.DEFAULT_PAGE_SIZE)
		doc.onFirstPage = self.myFirstPage
		doc.onNewPage = self.myLaterPages
		doc.leftMargin = 144
		elts = self.elts
		elts.insert(0, platypus.flowables.Spacer(4*inch, 4*inch)) # make space for title
		doc.build(self.elts)

	def processfile(self, fn, tofilename):
		f = type(fn) is type('') and open(fn) or fn
		lines = f.readlines()
		if tofilename is None:
			return self.process(lines, tofilename)
		else:
			self.process(lines, tofilename)

class DocStyle0HTML(DocStyle0PDF):
	def __init__(self):
		self.elts = []
		self.paragraph = []
		self.htmlfmt = '<html><header><title>%(title)s</title></header><body>%(body)s</body></html>'

	def finish(self,tofilename):
		if tofilename is not None:
			f = open(tofilename, "w")
		body = "\n".join(self.elts)
		D = {}
		D["title"] = self.Title
		D["body"] = body
		if tofilename is not None:
			out = self.htmlfmt % D
			f.write(out)
			f.close()
		else:
			return D

	def emit_paragraph(self, body):
		if self.mode=="normal":
			self.elts.append("<p>\n%s\n</p>\n"% body)
		elif self.mode=="preformatted":
			self.elts.append("<pre>\n%s\n</pre>\n"% body)
		else: raise "bad mode"

	def addHeader(self, header):
		self.elts.append("\n<h2>%s</h2>"%header)

	def addItem(self, item):
		self.elts.append("<b>%s</b><br>" % item)

	def addURL(self, URLdef):
		URLdef = URLdef.split()
		U=URLdef[0]
		if len(URLdef)<2:
			T = U
		else:
			T = URLdef[1:].join()
		self.paragraph.append(' <a href="%s"><b>%s</b></a>'%(U,T))

if __name__=='__main__':
	if len(sys.argv) != 2:
		print('Usage: simple_doc.py myfile.txt  will create HTML and PDF versions')
	else:
		filename = sys.argv[1]
		if os.path.isfile(filename):
			DoDualFormatPages([filename])
		else:
			print('%s not found' % filename)





#!/usr/bin/env python
#-------------------------------------------------------------------
# tarfile.py
#
# Module for reading and writing .tar and tar.gz files.
#
# Needs at least Python version 2.2.
#
# Please consult the html documentation in this distribution
# for further details on how to use tarfile.
# 
#-------------------------------------------------------------------
# Copyright (C) 2002 Lars Gustaebel <lars@gustaebel.de>
# All rights reserved.
#
# Permission  is  hereby granted,  free  of charge,  to  any person
# obtaining a  copy of  this software  and associated documentation
# files  (the  "Software"),  to   deal  in  the  Software   without
# restriction,  including  without limitation  the  rights to  use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies  of  the  Software,  and to  permit  persons  to  whom the
# Software  is  furnished  to  do  so,  subject  to  the  following
# conditions:
#
# The above copyright  notice and this  permission notice shall  be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS  IS", WITHOUT WARRANTY OF ANY  KIND,
# EXPRESS OR IMPLIED, INCLUDING  BUT NOT LIMITED TO  THE WARRANTIES
# OF  MERCHANTABILITY,  FITNESS   FOR  A  PARTICULAR   PURPOSE  AND
# NONINFRINGEMENT.  IN  NO  EVENT SHALL  THE  AUTHORS  OR COPYRIGHT
# HOLDERS  BE LIABLE  FOR ANY  CLAIM, DAMAGES  OR OTHER  LIABILITY,
# WHETHER  IN AN  ACTION OF  CONTRACT, TORT  OR OTHERWISE,  ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#copyright ReportLab Europe Limited. 2000-2012
"""Read from and write to tar format archives.
"""
from __future__ import print_function
__version__=''' $Id$ '''
# $Source: /rl_home/xxx/repository/rlextra/utils/tarfile.py,v $

version = "0.3.3"
__author__ = "Lars Gust\xe4bel (lars@gustaebel.de)"
__date__ = "$Date$"
__cvsid__ = "$Id$"
__credits__ = "Detlef Lannert for some early contributions"

#---------
# Imports
#---------
import sys
import os
try:
    import builtins
except ImportError:
    import __builtin__ as builtins
import shutil
import stat
import time

try:
    import grp, pwd
except ImportError:
    grp = pwd = None

#---------
# Exports
#---------
__all__ = ["TarFile", "TarInfo", "open", "gzopen",
           "REGTYPE", "AREGTYPE", "LNKTYPE",
           "SYMTYPE", "CHRTYPE", "BLKTYPE",
           "DIRTYPE", "FIFOTYPE", "CONTTYPE",
           "REGULAR_TYPES"]

#------------------------
# TAR specific constants
#------------------------
BLOCKSIZE = 512                 # length of processing blocks
RECORDSIZE = BLOCKSIZE * 20     # length of records (a product of BLOCKSIZE)
MAGIC = "ustar"                 # magic tar string
VERSION = "00"                  # version number

LENGTH_NAME = 100               # maximal length of a filename
LENGTH_LINK = 100               # maximal length of a linkname

REGTYPE = "0"                   # regular file
AREGTYPE = "\0"                 # regular file
LNKTYPE = "1"                   # link (inside tarfile)
SYMTYPE = "2"                   # symbolic link
CHRTYPE = "3"                   # character special device
BLKTYPE = "4"                   # block special device
DIRTYPE = "5"                   # directory
FIFOTYPE = "6"                  # fifo special device
CONTTYPE = "7"                  # contiguous file

GNUTYPE_LONGNAME = "L"          # GNUtar longnames
GNUTYPE_LONGLINK = "K"          # GNUtar longlink

#-------------------------------
# tarfile specific constants
#-------------------------------
SUPPORTED_TYPES = (REGTYPE, AREGTYPE, LNKTYPE,  # file types that tarfile
                   SYMTYPE, DIRTYPE, FIFOTYPE,  # can cope with.
                   CONTTYPE, GNUTYPE_LONGNAME,
                   GNUTYPE_LONGLINK)

REGULAR_TYPES = (REGTYPE, AREGTYPE, CONTTYPE)   # file types that represent
                                                # regular files

#-----------------------
# Some useful functions
#-----------------------
def nts(s):
    """Convert a null-terminated string buffer to
    a normal python string."""
    return s.split("\0", 1)[0]

def stn(s, maxlen=None):
    """Reverse method to nts. Pad with zero-chars up to
    maxlen (optional)."""
    if not maxlen: return s + "\0"
    return s + "\0" * (maxlen - len(s))

def calc_chksum(buf):
    """Calculate the checksum for an member's header. It's a simple addition
    of all bytes, treating the chksum field as if filled with spaces.
    buf is a 512 byte long string buffer which holds the header.
    """
    chk = 256                           # chksum field is treated as blanks,
                                        # so the initial value is 8 * ord(" ")
    for c in buf[:148]: chk += ord(c)   # sum up all bytes before chksum
    for c in buf[156:]: chk += ord(c)   # sum up all bytes after chksum
    return chk

def copyfileobj(fo1, fo2, length):
    """Copy length bytes from fileobj fo1 to fileobj fo2.
    """
    BUFSIZE = 65535
    blocks, remainder = divmod(length, BUFSIZE)
    for b in range(blocks):
        buf = fo1.read(BUFSIZE)
        if buf is None: raise IOError("end of file (EOF) reached")
        fo2.write(buf)

    buf = fo1.read(remainder)
    if buf is None: raise IOError("end of file (EOF) reached")
    fo2.write(buf)    

if os.sep != "/":
    def normpath(path):
        return os.path.normpath(path).replace(os.sep, "/")
else:
    normpath = os.path.normpath

class TarError(Exception):
    """Internal exception"""
    pass

#--------------------
# exported functions
#--------------------
def open(name, mode="r", fileobj=None):
    """Open a (uncompressed) tar archive for reading, writing
    or appending.
    """
    return TarFile(name, mode, fileobj)

def gzopen(gzname, gzmode="r", compresslevel=9, fileobj=None):
    """Open a gzip compressed tar archive for reading or writing.
    Appending is not allowed.
    """
    if gzmode == "a":
        raise RuntimeError("Appending to gzipped archive is not allowed")
    
    import gzip
    pre, ext = os.path.splitext(gzname)
    if ext == ".tgz": ext = ".tar"
    if ext == ".gz": ext = ""
    tarname = pre + ext
    mode = gzmode
    if "b" not in gzmode: gzmode += "b"
    if mode[0:1] == "w":
        if not fileobj:
            #AR 2.1 downgrade
            fileobj = builtins.open(gzname, gzmode)
            #fileobj = __builtin__.file(gzname, gzmode)
        #t = TarFile(tarname, mode, gzip.GzipFile(tarname, gzmode,
        #                                         compresslevel, fileobj))
        t = TarFile(tarname, mode[0:1], gzip.GzipFile(tarname, gzmode,
                                                 compresslevel, fileobj))
    else:
        t = TarFile(tarname, mode, gzip.open(gzname, gzmode, compresslevel))
    t._extfileobj = 0
    return t

def is_tarfile(name):
    """Return 1 if name points to a valid tar archive,
    else return 0.
    """
    buf = builtins.open(name, "rb").read(BLOCKSIZE)
    buftoinfo = TarFile.__dict__["_buftoinfo"]
    try:
        buftoinfo(None, buf)
    except ValueError:
        try:
            import gzip
            buf = gzip.open(name, "rb").read(BLOCKSIZE)
            buftoinfo(None, buf)
        except:
            try:
                import bz2
                buf = bz2.BZ2File(name, "r").read(BLOCKSIZE)
                buftoinfo(None, buf)
            except:
                return 0
    return 1
    
#------------------
# Exported Classes
#------------------
class TarInfo:
    """Informational class which represents a tar Header Block
    (POSIX 1003.1-1990).TarInfo objects are returned by
    TarFile.getinfo() and TarFile.getmembers() and are usually
    created internally.
    If you want to create a TarInfo object from the outside, you
    can use TarFile.gettarinfo() if the file already exists (recommended),
    or you can instanciate the class yourself.
    """

    name = ""               # member name (dirnames must end with '/')
    mode = 0o100666          # file permissions (default)
    uid = 0                 # user id
    gid = 0                 # group id
    size = 0                # file size
    mtime = 0               # modification time
    chksum = 0              # header checksum
    type = "0"              # member type, default REGTYPE
    linkname = ""           # link name
    uname = "user"          # user name
    gname = "group"         # group name
    devmajor = ""           #-+
    devminor = ""           # + not neccessary at this time
    prefix = ""             #-+
    
    offset = 0              # the tar header starts here
    offset_data = 0         # the REGTYPE file's data starts here

    def __init__(self, name=""):
        """Create a TarInfo object. name is the optional name
        of the member.
        """
        self.name = name
        
    def getheader(self):
        """Return a tar header as a 512 byte string.
        """
        # The following code was contributed by Detlef Lannert. Danke, Detlef.
        parts = []
        for value, fieldsize in (
                (self.name, 100),                       # file name
                ("%07o" % self.mode, 8),                # file permissions
                ("%07o" % self.uid, 8),                 # user id
                ("%07o" % self.gid, 8),                 # group id
                ("%011o" % self.size, 12),              # file size
                ("%011o" % self.mtime, 12),             # modification time
                ("        ", 8),                        # for checksum
                (self.type, 1),                         # file type
                (self.linkname, 100),                   # link name
                (MAGIC, 6),                             # magic string
                (VERSION, 2),                           # version number
                (self.uname, 32),                       # user name
                (self.gname, 32),                       # group name
                ):
            l = len(value)
            parts.append(value + (fieldsize - l) * "\0")
        
        buf = "".join(parts)
        chksum = calc_chksum(buf)
        buf = buf[:148] + "%07o" % chksum + buf[155:]
        buf += (512 - len(buf)) * "\0"
        self.buf = buf
        return buf
# class TarInfo


class TarFile:
    """Class representing a TAR archive on disk.
    """    
    debug = 0                   # may be set from 0 (no msgs) to 2 (all msgs)
    
    dereference = 1             # if true, add content of linked file to the
                                # tar file, else the link.
    fileobj = None

    def __init__(self, name=None, mode="r", fileobj=None):
        self.name = name
        
        if len(mode) > 1 or mode not in "raw":
            raise RuntimeError("mode must be either 'r', 'a' or 'w', " \
                                "not '%s'" % mode)
        self._mode = mode
        self.mode = {"r": "rb", "a": "r+b", "w": "wb"}[mode]
        
        if not fileobj:
            fileobj = builtins.open(self.name, self.mode)
            self._extfileobj = 0
        else:
            if self.name is None and hasattr(fileobj, "name"):
                self.name = fileobj.name
            if hasattr(fileobj, "mode"):
                self.mode = fileobj.mode
            self._extfileobj = 1
        self.fileobj = fileobj

        # Init datastructures
        self.members = []           # list of entires as TarInfo objects
        self.membernames = []       # names of members 
        self.chunks = [0]           # chunk cache
        self._loaded = 0            # flag if members have all been read
        self.offset = 0            # current position in the tarfile
        self.inodes = {}            # dictionary caching the inodes of archive
                                    # members already written

        if self._mode == "a":
            self.fileobj.seek(0)
            self._load()
            #self.fileobj.seek(self.chunks[-1])
            #DEL
            

        
    def _load(self):
        """Read through the entire tarfile and look for valid members.
        Is executed when e.g. getmembers() or getnames() is called,
        when there is a random access via read() or readstr().
        """
        while 1:
            tarinfo = next(self)
            if not tarinfo: break
        self._loaded = 1            
        #DEL self.fileobj.seek(0)
        return

    def _proc_gnulong(self, tarinfo, type):
        """Evaluate the two blocks that hold a longname.
        """
        name = None
        linkname = None
        
        buf = self.fileobj.read(BLOCKSIZE)
        if not buf: return None
        self.offset += BLOCKSIZE
        if type == GNUTYPE_LONGNAME:
            name = nts(buf)
        if type == GNUTYPE_LONGLINK:
            linkname = nts(buf)

        buf = self.fileobj.read(BLOCKSIZE)
        if not buf: return None
        tarinfo = self._buftoinfo(buf, tarinfo)
        if name is not None:
            tarinfo.name = name
        if linkname is not None:
            tarinfo.linkname = linkname
        self.offset += BLOCKSIZE
        return tarinfo
    
    def _buftoinfo(self, buf, tarinfo=None):
        """Return a TarInfo object filled with data
        provided by a 512 byte buffer.
        """
        if not tarinfo:
            tarinfo = TarInfo()
        tarinfo.name = nts(buf[0:100])
        tarinfo.mode = int(buf[100:108], 8)           
        tarinfo.uid = int(buf[108:116],8)             
        tarinfo.gid = int(buf[116:124],8)             
        tarinfo.size = int(buf[124:136], 8)          
        tarinfo.mtime = int(buf[136:148], 8)    
        tarinfo.chksum = int(buf[148:156], 8)
        tarinfo.type = buf[156:157]
        tarinfo.linkname = nts(buf[157:257])          
        tarinfo.uname = nts(buf[265:297])             
        tarinfo.gname = nts(buf[297:329])
        tarinfo.prefix = buf[345:500]
        return tarinfo

    def __next__(self):
        """Return the next member from the tarfile.
        Return None if the end of tarfile is reached.
        Can be used in a while statement and is used
        for Iteration (see __iter__()) and internally.
        """
        if not self.fileobj:
            raise IOError("Cannot access closed file")
        if self._mode not in "ra":
            raise IOError("File is not openend for reading.")

        # Read the next block ignoring empty or non-standard blocks.
        self.fileobj.seek(self.chunks[-1])
        while 1:
            buf = self.fileobj.read(BLOCKSIZE)
            if not buf: return None
            # Try to convert 512 byte block
            # to a TarInfo object.
            try: tarinfo = self._buftoinfo(buf)
            except ValueError:
                self.offset += BLOCKSIZE
                continue
            break

        # If the TarInfo object contains a GNUTYPE longname or longlink
        # statement, we must parse them first.
        if tarinfo.type in (GNUTYPE_LONGLINK, GNUTYPE_LONGNAME):
            tarinfo = self._proc_gnulong(tarinfo, tarinfo.type)

        # Add offsets to tarinfo
        tarinfo.offset = self.offset
        tarinfo.offset_data = self.offset + BLOCKSIZE

        # Proceed with the next block.
        self.offset += BLOCKSIZE

        if tarinfo.type in REGULAR_TYPES and tarinfo.size:
            # If header contains a size field,
            # the following n data blocks are skipped.
            blocks, remainder = divmod(tarinfo.size, BLOCKSIZE)
            if remainder: blocks += 1
            self.offset += blocks * BLOCKSIZE

        self.members.append(tarinfo)
        self.membernames.append(tarinfo.name)
        self.chunks.append(self.offset)       # next member's offset
        return tarinfo

    def _create_gnulong(self, name, type):
        """Insert a GNU longname/longlink member into a tarfile.
        It consists of a usual tar header, with the length
        of the filename as size, followed by data blocks (normally 1),
        which contain the filename as a null terminated string.
        """
        tarinfo = TarInfo()
        tarinfo.name = "././@LongLink"
        tarinfo.type = type
        tarinfo.mode = 0
        self.writestr(tarinfo, name + "\0")

    def getmembers(self):
        """Return a list of all members in tarfile (as TarInfo objects).
        """
        if not self._loaded: self._load()   # if we want to obtain a list of
                                            # all members, we first have to
                                            # read through the tar file.
        return self.members

    def getnames(self):
        """Return a list of names of all members in tarfile.
        """
        if not self._loaded: self._load()
        return self.membernames

    def __iter__(self):
        """Provide an iterator object for use in a for statement.
        Requires Python >= 2.2
        """
        if self._loaded:
            # All members read, iterate over internal list.
            return self.members.__iter__()
        else:
            return TarIter(self)

    def readstr(self, member):
        """Extract `member' and return its bytes. member may be a filename
        or a TarInfo instance.
        """
        if not self.fileobj: raise IOError("Cannot access closed file")
        if self._mode != "r":
            raise IOError("Cannot read from tar archive opened for writing")

        if isinstance(member, TarInfo):
            tarinfo = member
        else:
            tarinfo = self.getinfo(member)

        if tarinfo.type in REGULAR_TYPES:
            self.fileobj.seek(tarinfo.offset_data)
            return self.fileobj.read(tarinfo.size)
        elif tarinfo.type == LNKTYPE:
            linkname = tarinfo.linkname
            return self.readstr(self._getmember(linkname, tarinfo))
        else:
            return ""

    def read(self, member, path=""):
        """Extract `member' from the tarfile and write
        it to path (optional). member may be a filename or a TarInfo instance.
        """
        if not self.fileobj: raise IOError("Cannot access closed file")
        if self._mode != "r": raise IOError("Cannot read from write-mode file")

        if isinstance(member, TarInfo):
            tarinfo = member
        else:
            tarinfo = self.getinfo(member)

        if self.debug > 1: print(tarinfo.name, end=' ')
        try:
            self._extract_member(tarinfo, os.path.join(path, tarinfo.name))
        except TarError as e:
            if self.debug > 1:
                print()
                print("tarfile: '%s'" % e)
            return
        if self.debug > 1: print()

    def _extract_member(self, tarinfo, targetpath):
        """Extract the TarInfo object tarinfo to a pysical
        file called targetpath.
        """
        # Fetch the TarInfo object for the given name
        # and build the destination pathname, replacing
        # forward slashes to platform specific separators
        if targetpath[-1:] == "/": targetpath = targetpath[:-1]
        targetpath = os.path.normpath(targetpath)

        if tarinfo.type not in SUPPORTED_TYPES:
            raise TarError("Unsupported type `%s'" % tarinfo.type)

        # If the member is a directory, we are so nice
        # and create all upper directories on the fly.
        # os.makedirs() does a good job for that.
        if tarinfo.type == DIRTYPE:
            try: os.makedirs(targetpath)
            except (IOError, OSError) as e:
                raise TarError("Could not create %s" % tarinfo.name)

        # The member is regular file, resp. a contiguous file
        # which we treat like a regular one.
        # At first, all upper directories are created,
        # then we seek to the offset in the tarfile, where the
        # data locates and write it to the destination file.
        if tarinfo.type in REGULAR_TYPES:
            try:
                os.makedirs(os.path.dirname(targetpath))
            except (IOError, OSError) as e:
                pass
            try:
                self.fileobj.seek(tarinfo.offset_data)
                target = builtins.open(targetpath, "wb")
                copyfileobj(self.fileobj, target, tarinfo.size)
                target.close()
            except (IOError, OSError) as e:
                raise TarError(e)

        # The member is either a symbolic link or a hardlink
        # (a link inside the tarfile).
        # GNUtar behaviour is imitated. That means, if the link cannot
        # be created (because of an error or lacking support by filesystem)
        # we try to resolve the link. First we look for a valid member in the
        # tar archive, second we look in the filesystem.
        if tarinfo.type in (SYMTYPE, LNKTYPE):
            linkpath = tarinfo.linkname
            linkpath = os.path.normpath(linkpath)
            if self.debug > 1: print("->", linkpath, end=' ')

            created = 1
            if tarinfo.type == SYMTYPE:
                try: os.symlink(linkpath, targetpath)
                except (AttributeError, IOError, OSError) as e: created = 0
            elif tarinfo.type == LNKTYPE:
                try: os.link(linkpath, targetpath)
                except (AttributeError, IOError, OSError) as e: created = 0
                    
            if not created:
                linkpath = os.path.join(os.path.dirname(tarinfo.name),
                                        tarinfo.linkname)
                linkpath = os.path.normpath(linkpath)
                linkpath = normpath(linkpath)
                try:
                    self._extract_member(self.getinfo(linkpath), targetpath)
                except (IOError, OSError, KeyError) as e:
                    linkpath = os.path.normpath(linkpath)
                    try:
                        shutil.copy2(linkpath, targetpath)
                    except (IOError, OSError) as e:
                        raise TarError("Link could not be created")

        # The member is a fifo, so if the platform supports
        # fifos, we create one.
        if tarinfo.type in (FIFOTYPE,):
            if hasattr(os, "mkfifo"):
                try:
                    os.mkfifo(targetpath)
                except (IOError, OSError) as e:
                    raise TarError(e)
            else:
                raise TarError("fifo not supported by os")

        # The file has now been extracted. There are still some
        # things to be done.
        # All following tasks are skipped if the file was a symbolic
        # link.

        # Here we set the file's last modification time.
        # XXX If file is a directory, we do not set its mtime
        # because in most of the cases, it is reset to
        # current time when accessing files underneath.
        # Seems to make problems under win32 anyway.
        # (could possibly be resolved by an untar() method)
        if tarinfo.type not in (DIRTYPE, SYMTYPE):
            try:
                os.utime(targetpath, (tarinfo.mtime, tarinfo.mtime))
            except (IOError, OSError) as e:
                if self.debug > 2: print("utime '%s'" % e)

        # Here we set the ownership for the file.
        # We must be logged in as root. So, we
        # look for the owner on our system, and
        # if found, give it to him, else it's ours.
        if tarinfo.type != SYMTYPE and pwd and os.getgid() == 0:
            try: g = grp.getgrnam(tarinfo.gname)[2]
            except KeyError:
                try: g = grp.getgrgid(tarinfo.gid)[2]
                except KeyError: g = os.getgid()
            try: u = pwd.getpwnam(tarinfo.uname)[2]
            except KeyError:
                try: u = pwd.getpwuid(tarinfo.uid)
                except KeyError: u = os.getuid()
            try:
                os.chown(targetpath, u, g)
            except (IOError, OSError) as e:
                if self.debug > 2: print("chown '%s'" % e)

        # Here we set the file's permissions.
        if tarinfo.type != SYMTYPE:
            try:
                os.chmod(targetpath, tarinfo.mode)
            except (IOError, OSError) as e:
                if self.debug > 2: print("chmod '%s'" % e)
    
    def writestr(self, tarinfo, data="", fileobj=None):
        """Add the string buffer `data' to the tarfile.
        File information is passed by the tarinfo object.
        fileobj is used internally, for low memory consuming copying.
        """
        if not self.fileobj: raise IOError("Cannot access closed file")
        
        # If fileobj is not given, we write `data' to the archive.
        # So, the filesize is the length of the data. If fileobj
        # is given, tarinfo.size must have already been set.
        if not fileobj: tarinfo.size = len(data)

        # First, write out the tar header.
        try: self.fileobj.seek(self.chunks[-1])
        except IOError: pass
        header = tarinfo.getheader()
        self.fileobj.write(header)
        self.offset += BLOCKSIZE

        # If there's data to follow, we append it.        
        if data:
            self.fileobj.write(data)
            blocks, remainder = divmod(tarinfo.size, BLOCKSIZE)
            if remainder > 0:
                self.fileobj.write("\0" * (BLOCKSIZE - remainder))
                blocks += 1
            self.offset += blocks * BLOCKSIZE
        elif fileobj:
            copyfileobj(fileobj, self.fileobj, tarinfo.size)
            blocks, remainder = divmod(tarinfo.size, BLOCKSIZE)
            if remainder > 0:
                self.fileobj.write("\0" * (BLOCKSIZE - remainder))
                blocks += 1
            self.offset += blocks * BLOCKSIZE

        self.members.append(tarinfo)
        self.membernames.append(tarinfo.name)
        self.chunks.append(self.offset)

    def write(self, name, arcname=None, recursive=1):
        """Add a file or a directory to the tarfile.
        Directory addition is recursive by default.
        """
        if not self.fileobj: raise IOError("Cannot access closed file")
        if self._mode == "r":
            raise IOError("Cannot write to a read-mode file")

        if not arcname: arcname = name        

        # Skip if somebody tries to archive the archive...
        if os.path.abspath(name) == os.path.abspath(self.name):
            if self.debug > 2: print(name, "(skipped)")
            return

        # Special case: The user wants to add the current working directory.
        if name == ".":
            if recursive:
                if arcname == ".": arcname = ""
                for f in os.listdir("."):
                    self.write(f, os.path.join(arcname, f))
            return

        if self.debug > 1: print(name)

        # Create a TarInfo object from the file.
        tarinfo = self.gettarinfo(name, arcname)

        # Now we must check if the strings for filename
        # and linkname fit into the posix header. (99 chars + "\0" for each)
        # If not, we must create GNUtar specific extra headers.
        # If both filename and linkname are too long,
        # the longlink is first to be written out.
        if len(tarinfo.linkname) >= LENGTH_LINK - 1:
            self._create_gnulong(tarinfo.linkname, GNUTYPE_LONGLINK)
            tarinfo.linkname = tarinfo.linkname[:LENGTH_LINK -1]
        if len(tarinfo.name) >= LENGTH_NAME - 1:
            self._create_gnulong(tarinfo.name, GNUTYPE_LONGNAME)
            tarinfo.name = tarinfo.name[:LENGTH_NAME - 1]


        # In the end, we append the tar header and data
        # to the archive. If the file is a directory
        # we recurse its subdirectories (default).
        if tarinfo.type == REGTYPE:
            addf = builtins.open(name, "rb")        
            self.writestr(tarinfo, fileobj = addf)
            addf.close()

        if tarinfo.type in (LNKTYPE, SYMTYPE, FIFOTYPE):
            tarinfo.size = 0
            self.writestr(tarinfo, "")

        if tarinfo.type == DIRTYPE:
            self.writestr(tarinfo, "")
            if recursive: 
                for f in os.listdir(name):
                    self.write(os.path.join(name, f), os.path.join(arcname, f))

    def gettarinfo(self, name, arcname=None):
        """Create a TarInfo object from an existing file named
        `name'. Optional `arcname' defines the name under which
        the file shall appear in the archive. It defaults to `name'
        """
        # Building the name of the member in the archive.
        # Backward slashes are converted to forward slashes,
        # Absolute paths are turned to relative paths.
        if not arcname: arcname = name
        arcname = normpath(arcname)
        drv, arcname = os.path.splitdrive(arcname)
        while arcname[0:1] == "/": arcname = arcname[1:]

        # Now, we are about to fill the TarInfo object with
        # information specific for the file.
        tarinfo = TarInfo()

        # Use os.stat or os.lstat, depending on platform
        # and if symlinks shall be resolved.
        if hasattr(os, "lstat") and not self.dereference:
            osstat = os.lstat
        else:
            osstat = os.stat
            
        statres = osstat(name)
        linkname = ""

        # Here we test, which file type it is.
        stmd = statres[stat.ST_MODE]
        
        if stat.S_ISREG(stmd):                  # a regular file?
            inode = (statres[stat.ST_INO], statres[stat.ST_DEV], statres[stat.ST_MTIME])
            if inode in list(self.inodes.keys()):
                # Is it a hardlink to an already
                # archived file?
                type = LNKTYPE
                linkname = self.inodes[inode]
            else:
                # The inode is added only if its valid.
                # For win32 it is always 0.
                # So always self.inodes = {}
                type = REGTYPE
                if inode[0]: self.inodes[inode] = arcname
                
        elif stat.S_ISDIR(stmd):                # a directory?
            type = DIRTYPE
            if arcname[-1:] != "/": arcname += "/"

        elif stat.S_ISFIFO(stmd):               # a fifo?
            type = FIFOTYPE

        elif stat.S_ISLNK(stmd):                # a symlink?
            type = SYMTYPE
            linkname = os.readlink(name)

        else:
            if self.debug > 1: print("'%s' has an unsupported type" % name)
            return

        # Fill the TarInfo with all information we can get.
        #
        tarinfo.name = arcname
        tarinfo.mode = stmd
        tarinfo.uid = statres[stat.ST_UID]
        tarinfo.gid = statres[stat.ST_GID]
        tarinfo.size = statres[stat.ST_SIZE]
        tarinfo.mtime = statres[stat.ST_MTIME]
        tarinfo.type = type
        tarinfo.linkname = linkname
        if pwd: tarinfo.uname = pwd.getpwuid(tarinfo.uid)[0]
        if grp: tarinfo.gname = grp.getgrgid(tarinfo.gid)[0]

        return tarinfo        

    def getinfo(self, name):
        """Return a TarInfo object for file `name'.
        """
        # If the name is not found in the current
        # list of member's names, we force loading
        # the entire table of contents (which actually
        # does not exist).

        if name not in self.membernames and not self._loaded:
            self._load()
        if name not in self.membernames:
            raise KeyError("filename `%s' not found in tar archive" % name)

        return self._getmember(name)
        # DEL
        #for i in xrange(len(self.membernames) - 1, -1, -1):
        #    if name == self.membernames[i]:
        #        return self.members[i]

    def _getmember(self, name, tarinfo=None):
        """Find an archive member by name from bottom
        to top.
        If tarinfo is given, it is used as the starting point.
        """
        if not tarinfo:
            end = len(self.members)
        else:
            end = self.members.index(tarinfo)
            
        for i in range(end - 1, -1, -1):
            if name == self.membernames[i]:
                return self.members[i]
    
    def list(self):
        """Print a formatted listing of tarfile's contents to stdout.
        """
        print("%-46s %19s %12s" % ("File Name", "Modified    ", "Size"))
        for tarinfo in self.getmembers():
            date = "%d-%02d-%02d %02d:%02d:%02d" \
                   % time.gmtime(tarinfo.mtime)[:6]

            name = tarinfo.name
            if len(name) > 46: name = name[:20] + "[...]" + name[-21:]

            size = str(tarinfo.size)
            if tarinfo.type not in REGULAR_TYPES:
                size = ""

            print("%-46s %s %12s" % (name, date, size))

    def close(self):
        """Close the TarFile Object and do some cleanup.
        """
        if self.fileobj:
            if self._mode in "aw":
                # fill up the end with zero-blocks
                # (like option -b20 for tar does)
                blocks, remainder = divmod(self.offset, RECORDSIZE)
                if remainder > 0:
                    self.fileobj.write("\0" * (RECORDSIZE - remainder))

            if not self._extfileobj:
                self.fileobj.close()
            self.fileobj = None
# class TarFile

#---------------------------------------------
# zipfile compatible TarFile class
#
# for details consult zipfile's documentation
#---------------------------------------------
TAR_PLAIN = 0           # zipfile.ZIP_STORED
TAR_GZIPPED = 8         # zipfile.ZIP_DEFLATED
class TarFileCompat:
    """TarFile class in accordance with standard module zipfile.
    """
    def __init__(self, file, mode="r", compression=TAR_PLAIN):
        if compression == TAR_PLAIN:
            self.tarfile = open(file, mode)
        elif compression == TAR_GZIPPED:
            self.tarfile = gzopen(file, mode)
        else:
            raise IOError("unknown compression constant")
        if mode[0:1] == "r":
            import time
            members = self.tarfile.getmembers()
            for i in range(len(members)):
                m = members[i]
                m.filename = m.name
                m.file_size = m.size
                m.date_time = time.gmtime(m.mtime)[:6]
    def namelist(self):
        return [m.name for m in self.infolist()]
    def infolist(self):
        return [m for m in self.tarfile.getmembers() if m.type in REGULAR_TYPES]
    def printdir(self):
        self.tarfile.list()
    def testzip(self): return
    def getinfo(self, name):
        return self.tarfile.getinfo(name)
    def read(self, name):
        return self.tarfile.readstr(self.tarfile.getinfo(name))
    def write(self, filename, arcname=None, compress_type=None):
        self.tarfile.write(filename, arcname)
    def writestr(self, zinfo, bytes):
        import calendar
        zinfo.name = zinfo.filename
        zinfo.size = zinfo.file_size
        zinfo.mtime = calendar.timegm(zinfo.date_time)
        self.tarfile.writestr(zinfo, bytes)
    def close(self):
        self.tarfile.close()
#class TarFileCompat    

class TarIter:
    """Iterator Class. Enables the use of TarFile
    in a for statement. This needs Python >= 2.2.
    
    for tarinfo in TarFile(...):
        suite...
    """
    def __init__(self, tarfile):
        """Construct TarIter object.
        """
        self.tarfile = tarfile
    def __iter__(self):
        """Return iteration object, in this case self.
        """
        return self
    def __next__(self):
        """Return the next item using TarFile's next() method.
        When all members have been read, set TarFile as _loaded.
        """
        tarinfo = next(self.tarfile)
        if not tarinfo:
            self.tarfile._loaded = 1
            raise StopIteration
        return tarinfo
# class TarIter

if __name__ == "__main__":
    # a "light-weight" implementation of GNUtar ;-)
    usage = """
Usage: %s [options] [files]

-h      display this help message
-c      create a tarfile
-r      append to an existing archive
-x      extract archive
-t      list archive contents
-f FILENAME
        use archive FILENAME, else STDOUT (-c)
-z      filter archive through gzip
-C DIRNAME
        with opt -x:     extract to directory DIRNAME
        with opt -c, -r: put files to archive under DIRNAME
-q      quiet

wildcards *, ?, [seq], [!seq] are accepted.
    """ % sys.argv[0]
    
    import getopt, glob
    try:
        opts, args = getopt.getopt(sys.argv[1:], "htcrzxf:C:q")
    except getopt.GetoptError as e:
        print()
        print("ERROR:", e)
        print(usage)
        sys.exit(0)
    
    file = None
    mode = None
    dir = ""
    comp = 0
    debug = 3
    for o, a in opts:
        if o == "-t": mode = "l"        # list archive
        if o == "-c": mode = "w"        # write to archive
        if o == "-r": mode = "a"        # append to archive
        if o == "-x": mode = "r"        # extract from archive
        if o == "-f": file = a          # specify filename else use stdout
        if o == "-C": dir = a           # change to dir
        if o == "-z": comp = 1          # filter through gzip
        if o == "-q": debug = 0         # quiet mode
        if o == "-h":                   # help message
            print(usage)
            sys.exit(0)

    if not mode:
        print(usage)
        sys.exit(0)

    if comp:
        func = gzopen
    else:
        func = open
    
    if not file:
        if mode != "w":
            print(usage)
            sys.exit(0)
        debug = 0

        # If under Win32, set stdout to binary.
        try:
            import msvcrt
            msvcrt.setmode(1, os.O_BINARY)
        except: pass

        tarfile = func("sys.stdout.tar", mode, sys.stdout)
    else:
        if mode == "l":
            tarfile = func(file, "r")
        else:
            tarfile = func(file, mode)
        
    
    tarfile.debug = debug

    if mode == "r":
        while 1:
            tarinfo = next(tarfile)
            if not tarinfo: break
            tarfile.read(tarinfo, dir)
            
    elif mode == "l":
        tarfile.list()
    else:
        for arg in args:
            files = glob.glob(arg)
            for f in files:
                tarfile.write(f, dir)

    tarfile.close()                



#copyright ReportLab Europe Limited. 2000-2012
# helpers for making html forms for updating db records.

__version__=''' $Id$ '''


### should check html quoting everywhere someday! (eg in hidden fields)
### should generalize to support non unary foreign keys

def timedict():
    from time import localtime, time
    (year, month, day, hour, mm, ss, ms, x, y) = localtime(time())
    return {"year":year, "month":month, "day":day, "hour": hour, "minute":mm}

nowdict = timedict()

def dayselect(name="day", selected=None, values=list(range(1,32)), names=list(range(1,32)),width=None, onchange=None):
    if selected is None:
        selected = nowdict["day"]
    if onchange and width:
        L = ["""<select name="%s" style="width: %s" onchange="%s">""" % (name, width, onchange)]
    elif width:
        L = ["""<select name="%s" style="width: %s">""" % (name, width)]
    else:
        L = ["""<select name="%s">""" % name]
    for i in range(len(values)):
        namei = names[i]
        valuei = values[i]
        li = """   <option value="%s">%s</option>""" % (valuei, namei)
        if selected==valuei:
            li = """      <option selected value="%s">%s</option>""" % (valuei, namei)
        L.append(li)
    L.append("</select>")
    return '\n'.join(L)

nowdayselect = dayselect()

def select(name, names, values, selected=None,width=None, onchange=None):
    """<select...> html element"""
    if len(names)!=len(values):
        raise ValueError("names and values of different len")
    if selected is not None and selected not in values:
        raise ValueError("selected %s not in values: %s..." % (repr(selected), repr(values[0])))
    return dayselect(name=name, names=names, values=values, selected=selected, width=width, onchange=onchange)

def monthselect(name="month", selected=None):
    if selected is None:
        selected = nowdict["month"]
    values = list(range(1,12))
    import calendar
    names = calendar.month_name[1:]
    return dayselect(name, selected, values, names)

nowmonthselect = monthselect()

def yearselect(name="year", selected=None, past=5, forward=5):
    if selected is None:
        selected = nowdict["year"]
    values = list(range(selected-past, selected+forward+1))
    return dayselect(name, selected, values, values)

def hiddenFields(pairs):
    L = []
    for (name, value) in pairs:
        h = """<input type="hidden" name="%s" value="%s">""" % (name, value)
        L.append(h)
    return "\n".join(L)

class DateField:

    def __init__(self, month=None, day=None, year=None, epoch=None):
        import time
        if epoch is not None:
            (year, month, day, hour, mm, ss, ms, x, y) = time.localtime(epoch)
        else:
            if year is None:
                year = nowdict["year"]
            if day is None:
                day = nowdict["day"]
            if month is None:
                month = nowdict["month"]
            epoch = time.mktime( (year,month,day, 0, 0, 0, 0, 0, 0) )
        #print year, month, day, epoch, "<br>"
        self.year, self.month, self.day, self.epoch = year, month, day, epoch
        #print "epoch", epoch, "<br>"
    def htmlData(self):
        from calendar import month_name
        nameofmonth = month_name[self.month]
        return "%s %s %s" % (self.day, nameofmonth, self.year)
    def formField(self, prefix, table, join=1, separator=" "):
        ys = yearselect(prefix+"_yr", self.year)
        ms = monthselect(prefix+"_mn", self.month)
        ds = dayselect(prefix+"_dy", self.day)
        L = [ys, ms, ds]
        if not join: return L
        return separator.join(L)
    def PDFdata(self):
        return self.htmlData() # default
    def hiddenFormField(self, prefix):
        return hiddenFields( [(prefix+"_yr", self.year), (prefix+"_mn", self.month), (prefix+"_dy", self.day)] )
    def initFromDataValue(self, d):
        self.__init__(epoch=d)
    def initFromCGI(self, prefix, cgiform):
        D = {}
        for (suffix, kw) in [("_yr", "year"), ("_mn", "month"), ("_dy", "day")]:
            f = cgiform[prefix+suffix]
            v = f.value
            D[kw] = int(v)
        #print D, "<br>"
        self.__init__(*(), **D)
    def testCGI(self, prefix, cgiform):
        for suffix in ("_yr", "_mn", "_dy"):
            try:
                cgiform[prefix+suffix]
            except:
                #print "test failed", prefix, suffix, "<br>"
                return 0
        return 1
    def dataValue(self):
        return self.epoch
    sqlDataType = "float"
    def sqlLiteral(self):
        return str(self.epoch)

class SmallStringField(DateField):
    sqlDataType = "varchar"
    def __init__(self):
        self.value = ""
    def htmlData(self):
        return self.value
    def formField(self, name, table):
        return """<input type="text" name="%s" value="%s">""" % (name, self.value)
    def initFromCGI(self, prefix, cgiform):
        try:
            f = cgiform[prefix]
        except:
            self.value = "" # no value default
        else:
            v =  f.value
            # hack out anomalies!
            v = v.split("\n")
            v = list(map(str.strip, v))
            v = "\n".join(v)
            self.value = v
    def testCGI(self, prefix, cgiform):
        try:
            cgiform[prefix]
        except:
            return 0
        else:
            return 1
    def initFromDataValue(self, d):
        self.value = d
    def dataValue(self):
        return self.value
    def sqlLiteral(self):
        #print self.value
        v2 = self.value.replace("'","''")
        return "'%s'" % v2
    def hiddenFormField(self, prefix):
        return hiddenFields( [(prefix, self.value)] )

class ForeignKey(SmallStringField):
    "not implemented"

class LargeStringField(SmallStringField):
    def htmlData(self):
        "truncated"
        return """<pre>%s</pre>""" % self.value
    def PDFdata(self):
        return self.value # default
    def formField(self, name, table):
        #print self.value
        return """<textarea name="%s" cols=60 rows=5>%s</textarea>"""%(name, self.value)

class Table:

    def __init__(self, name, PrimaryKeySeq, fieldclasspairs):
        self.name = name
        fieldclassmap = {}
        fieldsequence = []
        for (name, klass) in fieldclasspairs:
            if name in fieldclassmap:
                raise ValueError("field name declared twice %s" % name)
            fieldclassmap[name] = klass
            fieldsequence.append(name)
        for pkf in PrimaryKeySeq:
            if pkf not in fieldclassmap:
                raise ValueError("primary key field not declared %s" % pkf)
        self.fieldclassmap = fieldclassmap
        self.fieldsequence = fieldsequence
        self.primarykeysequence = tuple(PrimaryKeySeq)
        self.actions = []
        #self.makeDeleteAction()
    def CGIfields(self, form):
        result = {}
        for (name, klass) in list(self.fieldclassmap.items()):
            fi = klass()
            test = fi.testCGI(name, form)
            if test:
                fi.initFromCGI(name, form)
                result[name] = fi.dataValue()
        return result
    def makeDeleteAction(self, actionname, method="post", miscdata=""):
        miscdata = miscdata+hiddenFields([("table_action", "delete"), ("table_name", self.name)])
        deleteaction = addedaction_notranslate("delete", self.fieldsequence, actionname, miscdata, method)
        self.add_action(deleteaction)
    def add_action(self, action):
        self.actions.append(action)
    def createStatement(self):
        L = ["create table %s (\n" % self.name]
        a = L.append
        fcm = self.fieldclassmap
        started = 0
        for f in self.fieldsequence:
            if started: a(",\n")
            started = 1
            k = fcm[f]
            ty = k.sqlDataType
            a("    %s %s" % (f, ty))
        a("\n)")
        return ' '.join(L)
    def primaryIndex(self):
        n = self.name
        L = ["create unique index PK_%s on %s (" % (n,n)]
        a = L.append
        started = 0
        for k in self.primarykeysequence:
            if started: a(",")
            started = 1
            a(k)
        a(")")
        return ' '.join(L)
    # XXX should add other index support
    insertPrefixCached = None
    def insertPrefix(self):
        if self.insertPrefixCached: return self.insertPrefixCached
        L = ["insert into %s ( " % self.name]
        a = L.append
        started = 0
        for f in self.fieldsequence:
            if started: a(",")
            started = 1
            a(f)
        a(") values ")
        r = self.insertPrefixCached = ' '.join(L)
        return r
    def insert(self, dictionary):
        ### !!! should optimize
        L = [self.insertPrefix()]
        a = L.append
        fieldclassmap = self.fieldclassmap
        started = 0
        a("\n   (")
        #for (a,b) in dictionary.items(): print a,b,"<br>"
        for f in self.fieldsequence:
            if started: a(",")
            started = 1
            fc = fieldclassmap[f]
            fi = fc()
            v = dictionary[f]
            fi.initFromDataValue(v)
            lit = fi.sqlLiteral()
            a(lit)
        a(")")
        return ' '.join(L)
    def whereMatch(self, dictionary):
        L = ["where\n   "]
        a = L.append
        fieldclassmap = self.fieldclassmap
        started = 0
        for k in list(dictionary.keys()):
            if started: a("\n   and")
            started = 1
            fc = fieldclassmap[k]
            fi = fc()
            v = dictionary[k]
            fi.initFromDataValue(v)
            lit = fi.sqlLiteral()
            a("%s=%s" % (k,lit))
        return ' '.join(L)
    def delWhere(self, dictionary):
        return "delete from %s\n%s" %(self.name, self.whereMatch(dictionary))
    def selWhere(self, dictionary=None):
        order_by = ", ".join(self.primarykeysequence)
        select_list = ", ".join(self.fieldsequence)
        whereclause = ""
        if dictionary: whereclause = self.whereMatch(dictionary)
        return "select %s\nfrom %s\n%s\norder by %s" %(select_list, self.name, whereclause, order_by)
    # no modify support yet
    def insertForm(self, action, miscdata, method="post", dictionary=None):
        # use misc data to insert, eg standard hidden fields
        if not dictionary: dictionary = {}
        L = []
        a = L.append
        a("""<form action="%s" method="%s">""" % (action, method))
        fieldclassmap = self.fieldclassmap
        a(miscdata)
        for f in self.fieldsequence:
            fc = fieldclassmap[f]
            fi = fc()
            v = dictionary.get(f, None)
            if v is not None:
                fi.initFromDataValue(v)
            ff = fi.formField(f, self)
            a("%s: %s<br>" % (f, ff))
        a("""<input type="hidden" name="table_action" value="insert">""")
        a("""<input type="hidden" name="table_name" value=%s>""" % self.name)
        a("""<input type="submit" value="add entry">""")
        a("""</form>""")
        return "\n".join(L)
    def whereTable(self, cursor, dictionary=None):
        L = []
        a = L.append
        wherestatement = self.selWhere(dictionary)
        a("<pre>")
        a(wherestatement)
        cursor.execute(wherestatement)
        a("</pre>")
        a("<hr>")
        a("<table>\n")
        fieldsequence = self.fieldsequence
        fieldclassmap = self.fieldclassmap
        actions = self.actions
        a("<tr>\n")
        for n in fieldsequence:
            a("<th> %s </th>"%n)
        a("</tr>\n")
        for resultrow in cursor.fetchall():
            a("<tr>")
            D = {}
            for (name, value) in map(None, fieldsequence, resultrow):
                fc = fieldclassmap[name]
                fi = fc()
                fi.initFromDataValue(value)
                D[name] = fi
                hd = fi.htmlData()
                a("<td> %s </td>" % hd)
            for act in actions:
                doit = act(D)
                a("<td> %s </td>" % doit)
            a("</tr>\n")
        a("</table>")
        return " ".join(L)
    def pdfrows(self, cursor, dictionary):
        "return list of dictionary or pdf formattable data for selection"
        result = []
        wherestatement = self.selWhere(dictionary)
        print(wherestatement)
        cursor.execute(wherestatement)
        fieldsequence = self.fieldsequence
        fieldclassmap = self.fieldclassmap
        resultrows = cursor.fetchall()
        print(len(resultrows), "rows")
        for resultrow in resultrows:
            D = {}
            for (name, value) in map(None, fieldsequence, resultrow):
                fc = fieldclassmap[name]
                fi = fc()
                fi.initFromDataValue(value)
                D[name] = fi.PDFdata()
            result.append(D)
        return result

def addedaction_notranslate(button, names, action, miscdata="", method="post"):
    D = {}
    for n in names:
        D[n] = n
    return addedaction(button, D, action, miscdata, method)

class addedaction:
    def __init__(self, button, nametranslation, action, miscdata="", method="post"):
        self.nametranslation = nametranslation
        self.button = button
        self.action = action
        self.method = method
        self.miscdata = miscdata
    def __call__(self, dictionary):
        # dictionary should be oldname to field *instances!*
        nametranslation = self.nametranslation
        action = self.action
        button = self.button
        method = self.method
        L = []
        a = L.append
        a("""<form action= "%s" method="%s">""" %(action, method))
        for oldname in list(nametranslation.keys()):
            newname = nametranslation[oldname]
            fieldvalue = dictionary[oldname]
            hidden = fieldvalue.hiddenFormField(newname)
            a(hidden)
        a(self.miscdata)
        a("""<input type="submit" value="%s">""" % button)
        a("""</form>""")
        return "\n".join(L)

nowyearselect = yearselect()

if __name__=="__main__":
   fn = "test.html"
   f = open(fn, "w")
   D = {}
   D["month"] = nowmonthselect
   D["year"] = nowyearselect
   D["day"] = nowdayselect
   f.write("""
   <form action="test.cgi">
   %(month)s %(day)s %(year)s<br>
   <input type="submit">
   </form>""" % D)
   f.close()
   print("wrote", fn)
   print("now creating database")
   fn = "test2.html"
   import sys
   sosave = sys.stdout
   ff = sys.stdout = open(fn, "w")
   print("<pre>")
   client = Table("client", ["handle"],
       [("handle", SmallStringField), ("contact", SmallStringField), ("started", DateField)])
   client.makeDeleteAction("delete")
   print();print(client.createStatement())
   print();print(client.primaryIndex())
   print();print(client.insertPrefix())
   D = {"handle": "woogie"}
   import time
   D2 = {"handle": "woogie", "contact": "poogie", "started": time.time()}
   print();print(client.insert(D2))
   print();print(client.whereMatch(D))
   print();print(client.delWhere(D))
   print();print(client.selWhere(D))
   print("</pre>")
   print(client.insertForm("action", "miscdata", "post", D))
   print("<hr>")
   import gadfly
   g = gadfly.gadfly()
   g.startup("dbtest", ".")
   c = g.cursor()
   c.execute(client.createStatement())
   c.execute(client.primaryIndex())
   c.execute(client.insert(D2))
   print(client.whereTable( c, dictionary=D))
   sys.stdout = sosave
   ff.close()
   print("wrote", fn, "for testing now finishing db init")
   c.execute("delete from client")



__code__='Gat=(s)6@U\'RuIZTe-[Fl.FWrG^S2]UcB+n6IRcNKlf\\&Z\\fGPKh[m!GAU%pgRR`d<%:*IBts#<g0_;_c8J7L1o.SM48\\I&kKC4#754=WSZ\\UK.fQ,l2dljY$OT.L\\%qOD!@%VC^bP?\'j=cT(5l*>=*Pa\\N(>p(GH,\'U!7DQ6[k5[^J4:cZ;E!c"Xi%j4:^_OYG!A=P(!r+5&\'u8,q56nh.GRa-M_!ApZiDl)PT)<jKrZH(`7WT\\VX?\\fhkMWiViq;1_I-JF>@WqjfA>pg%E!$=3>@bVW$H#D6(3s`EoeY@88`9rJ-`nCE3%DTI2YC*)+[$=hc4m0ZiUG>:?!YJ?\\Qf2r"C-E)"DpGcRLDLaMdJb/lC"N7X;s985lS:M._)e)!^peH>S?bT=c1&_,1q/6[(k:@ZF8Fu7WX=$NmscRb4*>QQk[uo(=at6D^m"!Ej"D$QPLRQ-eYq7;Suh\\`K"ZZ\\<XMUQ4;sd"dR/c<oLO^]iFO;UJ@MW:,UQV3_kQmfuas"njAn<k6tD8gCc/sA[*L3gX+.8IOUc<JfkNC1\'l*uI2HU1&q2rU+((KkWRbneZ;al_D&-A&-!<WjKAb7*m>GN1Jcc<jl&Q&OZVI:m9]hbkI^nKWps01_>e$Js`0!_$M/KIeEMVE,HY20<f(o\'2cG@>JVsIq,DL#);p=p:kmi.>-hL:iZ:rc`(B$-C&4B@+PAbk:V32WNOVbHam>O5roR9\\?15+\'kXmS)2eG-;iK#lcWn03,hlFdBR1\'!6VLYjRu"-Oj&amh(f`2p=Fc-Ij9iDRuhG?HMUHSXUQ5m=p>eb4PA@3D\\\\K];p/?pMffAG7X.,Wg96GjV,e/VHU!aA6GV>hn/#$\\r"J?4saf\\IT=LoE3%_+ZKmu+)P^RUmD\\<b?1Ds_modC$/_\'7YG#@i+h)2MoZYi(am4_9\\<WKnfO1=&.m8`\'#$]@*.O\\rAIFC2sh@LF.*SXI7G<o%ZgCRTXqc/Y(n^$uP2/On@JL\\jYqHK-1XD+!:[r_+E7:*[c@VUP7A:!E<so#u*@SGK1hjM2sHA)WHU676Y3hk/=*g[fQ7$\\C\'(86DjVdo`U")/D5bm)#?Rq^4G^\'R6YR1r<D;%qU+aO]dQ[p]ZIHXTbsP>6"2DV6M4JNILt-n"R>~>'




"test extracting info from pdf-forms"

pdfsource = "f1040ez.pdf"
storagefile = "f1040ez.storage"
formdump = "f1040ez.py"
pdfoutfile="f1040annotated.pdf"
import string, sys
from reportlab.pdfgen.canvas import Canvas

def annotate(pdfoutfile=pdfoutfile, pdfsource=pdfsource, formdump=formdump, storagefile=storagefile):
    canvas = Canvas(pdfoutfile)
    print("first get the formdata")
    formdatatext = string.strip(open(formdump).read())
    formdata = eval(formdatatext)
    print("now catch the form pages")
    from rlextra.pageCatcher import pageCatcher
    formnames = pageCatcher.storeForms(pdfsource, storagefile, all=1, verbose=1)
    pageCatcher.restoreForms(storagefile, canvas)
    npages = len(formnames)
    print(formnames)
    if len(formdata)!=npages:
        raise ValueError("number of forms and pages should match")
    # now annotate the form pages with the field info
    for pagenumber in range(npages):
        formname = formnames[pagenumber]
        canvas.doForm(formname)
        canvas.setStrokeColorRGB(0, 0.5, 1)
        canvas.setFillColorRGB(1, 1, 0)
        canvas.setFont("Courier", 5)
        formdatum = formdata[pagenumber]
        for field in formdatum:
            if field is None:
                print("Warning, field repeats!")
            elif "Rect" in field:
                # draw the rectangle
                rect = field["Rect"]
                #print rect
                (x1, y1, x2, y2) = rect
                canvas.rect(x1, y1, x2-x1, y2-y1, fill=1)
        canvas.setFillColorRGB(0.5, 0, 0)
        count = 0
        for field in formdatum:
            if field and "Rect" in field:
                rect = field["Rect"]
                (x1, y1, x2, y2) = rect
                canvas.saveState()
                canvas.translate(x1+1, y1+1)
                canvas.rotate(20)
                T = FT = None
                if "T" in field:
                    T = field["T"]
                if "FT" in field:
                    FT = field["FT"]
                canvas.setFillColorRGB(1,1,1)
                canvas.rect(0,0,25,3,fill=1,stroke=0)
                canvas.setFillColorRGB(0.5, 0, 0)
                canvas.drawString(0,0, "[%s][%s] %s:%s" %(pagenumber, count, T, FT))
                canvas.restoreState()
                count = count+1
        canvas.showPage()
    canvas.save()
    print("wrote", pdfoutfile)

if __name__=="__main__":
    annotate()

    


#example1_fillform
# uses a PDF as a form and fills in first page


try:
    from rlextra.pageCatcher.pageCatcher import storeForms, restoreForms
except ImportError:
    pass # running inside pageCatcher module?


data = """
1.7     24.5    Mr. Bond T. Owner
8.5     24.5    11-222-3344
11.5    24.5    Ms. With H. Agent
16.7    24.5    99-88-7777
1.7     23.4    10 Main St., Podunk OH 667788
11.5    23.4    1 Big Building, New York, NY 10011
9       22.5    Bernum & Runn, Inc 1999
15.3    22.5    10 Jan 2000
17.8    22.5    10 Dec 2000
1.7     21.7    (X)
1.7     20.8    F. Agent, Esquire
1.7     20.4    15 Central Av., Newark, NJ 11989
15.5    20.3    1999.90
18      20.3    0.00
15.5    19.5    1666.00
"""

def preprocess_form(pdffile,
                    storagefile):
    storeForms(pdffile, storagefile)
    print('preprocessed %s -> %s' % (pdffile, storagefile))
        

def do_test_form(storagefile, 
                 testfile, 
                 data=data):
    # make a canvas to draw on
    from reportlab.pdfgen import canvas
    canv = canvas.Canvas(testfile)
    # get the stored form layout
    names = restoreForms(storagefile, canv)
    # put the form layout on the current page
    canv.doForm(names[0])
    # fill the form in...
    canv.setFont("Helvetica", 10)
    from reportlab.lib.units import cm
    from reportlab.lib.colors import green
    canv.setFillColor(green)
    import string
    lines = string.split(string.strip(data), "\n")
    for l in lines:
        sline = string.split(l)
        x, y = float(sline[0]), float(sline[1])
        entry = string.join(sline[2:], " ")
        canv.drawString(x*cm, y*cm, entry)
    canv.showPage()
    canv.save()
    print('wrote filled form as %s' % testfile)

def run(preprocess_form=preprocess_form, do_test_form=do_test_form):
    preprocess_form('sample1.pdf','sample1.data')
    do_test_form('sample1.data','out1_fillform.pdf')
    

if __name__=="__main__":
    #print dir()
    run(preprocess_form, do_test_form)


def __bootstrap():
    import sys, zlib, marshal
    from reportlab.lib.rl_accel import asciiBase85Decode
    from reportlab.lib.utils import rl_exec
    mname = 'rlextra.rml2pdf.rml2pdf_%d%d%s' % (sys.version_info[:2]+('c' if not sys.flags.optimize else 'o',))
    m = {}
    rl_exec('import %s as m' % mname, m)
    m = m['m']
    c=sys.modules[__name__]
    N=dict([(a,getattr(c,a)) for a in '__file__ __name__ __path__ __package__'.split() if hasattr(c,a)])
    rl_exec(marshal.loads(zlib.decompress(asciiBase85Decode(m.__code__))),c.__dict__)
    c.__dict__.update(N)
    del sys.modules[mname]

__bootstrap()
del __bootstrap

if __name__=='__main__':
    import sys
    main(exe=1,fn=sys.argv[1:])



#copyright ReportLab Inc. 2000-2012
__version__=''' $Id$ '''
__all__=('URL2FSCacheKey','URL2FSCacheValue','URLValue','URL2FSCache',
        )
#generic url cacher
try:
    import urllib.request, urllib.error
    urllib_Request = urllib.request.Request
    urllib_urlopen = urllib.request.urlopen
    urllib_HTTPError = urllib.error.HTTPError
    del urllib.request, urllib.error
except ImportError:
    import urllib2
    urllib_Request = urllib2.Request
    urllib_urlopen = urllib2.urlopen
    urllib_HTTPError = urllib2.HTTPError
    del urllib2
    
import os, time, zlib
from reportlab.lib.utils import pickle
class URL2FSCacheKey(tuple):
    for i,x in enumerate("url".split()):
        exec("%s=property(lambda self: self[%d])" % (x,i))

class URL2FSCacheValue(tuple):
    for i,x in enumerate("path lastmod etag lastfetch".split()):
        exec("%s=property(lambda self: self[%d])" % (x,i))

class URLValue(str):
    def __new__(cls,value,info=None):
        self = str.__new__(cls,value)
        self._info = info
        return self

def recentLocalFile(path,tooOld=None):
    '''return True for files that exists and are less than tooOld'''
    return path and os.path.isfile(path) and tooOld is not None and os.stat(path)[8]<tooOld

class URL2FSCache:
    def __init__(self,path):
        self._lastfetch = 0
        self._path = path
        self.readCache()

    def readCache(self):
        try:
            cache = pickle.loads(zlib.decompress(open(self._path,'rb').read()))
            #print '_readCache', self._path, cache
        except:
            cache = {}
        self._cache = cache

    def writeCache(self):
        open(self._path,'wb').write(zlib.compress(pickle.dumps(self._cache)))
        #print 'writeCache',self._path, self._cache

    def readText(self,url,msgFunc=None):
        v = self._cache[url]
        f = open(v.path,'rb')
        text = f.read()
        now = time.time()
        f.close()
        self._cache[url] = URL2FSCacheValue(v[:3]+(now,))
        if msgFunc: msgFunc(' '.join(('cached',url,v.path)))
        return URLValue(text,'cached')

    def fetch(self, url, path, force=1, delay=0, msgFunc=None, tooOld=None):
        cache = self._cache
        req = urllib_Request(url)
        v = cache.get(url,None)
        if v and recentLocalFile(v.path,tooOld):
            if not force:
                return self.readText(url)

            v = cache[url]
            if v.etag: req.add_header('If-None-Match',v.etag)
            if v.lastmod: req.add_header('If-Modified-Since',v.lastmod)
            try:
                if delay>0: time.sleep(delay)
                res = urllib_urlopen(req)
            except urllib_HTTPError as e:
                if '304' not in str(e): raise
                #unchanged
                print('unchanged')
                return self.readText(url,msgFunc)
            if msgFunc: msgFunc(' '.join(('fresh',url)))
            self.delete(url)
        else:
            self.delete(url)
            if delay>0: time.sleep(delay)
            res = urllib_urlopen(req)
            if msgFunc: msgFunc(' '.join(('new',url)))
        text = res.read()
        if path: self.writeText(text,path,msgFunc)
        get = res.headers.dict.get
        now = time.time()
        cache[url] = URL2FSCacheValue((path,get('last-modified'),get('etag'),now))
        return URLValue(text,'fresh')

    def __getitem__(self,url):
        return self._cache[url]

    @staticmethod
    def writeText(text,path,msgFunc=None):
        dir = os.path.dirname(path)
        try:
            os.makedirs(dir)
        except:
            pass
        f = open(path,'wb')
        f.write(text)
        f.close()
        if msgFunc: msgFunc(' '.join(('wrote',path,'length',str(len(text)))))

    def setPath(self,url,path):
        v = self._cache[url]
        self._cache[url] = URL2FSCacheValue((path,)+v[1:])

    def delete(self,url):
        try:
            v = self._cache[url]
            if v.path:
                try:
                    os.remove(v.path)
                except OSError:
                    pass
            del self._cache[url]
        except KeyError:
            pass

def test():
    u=URL2FSCache('mycache.zpckl')
    print(u.fetch('http://svn.reportlab.com/testa/a.txt','testa/a.txt'))
    print(u.fetch('http://svn.reportlab.com/testb/b.txt','testb/b.txt'))
    u.writeCache()

if __name__=='__main__':
    test()



#example4_fourpage.py
#prints PDF four to a page.  Works best with A4 portrait files;
from reportlab.pdfgen import canvas
try:
    from rlextra.pageCatcher.pageCatcher import storeForms, restoreForms, fourPage
except ImportError:
    pass # running inside pageCatcher module?

def run():
    inputFileName = "sample2.pdf"
    outputFileName = "out4_fourpage.pdf"
    storageFile = "sample2.data"

    print("reversing", inputFileName, "into", outputFileName)

    storeForms(inputFileName, storageFile, all=1)
    print("stored all pages to", storageFile)
    #canv = canvas.Canvas(outputFileName)
    #formNames = restoreForms(storageFile, canv)

    fourPage(storageFile, outputFileName)

if __name__=='__main__':
    run()

    



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.charts.legends import Legend
from reportlab.graphics.widgets.markers import makeMarker
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String
from reportlab.graphics.charts.lineplots import LinePlot
from reportlab.lib.validators import Auto

class LegendDrawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self._add(self,LinePlot(),name='plot',validate=None,desc=None)
		self.plot.y                = 20
		self._add(self,String(200,180,"text"),name='title',validate=None,desc=None)
		self.title.fontSize   = 12
		self.title.textAnchor='middle'
		self.title.text       = 'Example 4 - auto legend'
		self.title.fontName   = 'Times-Bold'
		self._add(self,Legend(),name='legend',validate=None,desc=None)
		self.legend.boxAnchor='sw'
		self.legend.x              = 220
		self.legend.y              = 20
		self.legend.colorNamePairs = Auto(chart=self.plot)
		self.plot.lines[0].symbol        = makeMarker('Diamond')

if __name__=="__main__": #NORUNTESTS
	LegendDrawing().save(formats=['pdf'],outDir='.',fnRoot=None)



"""
pageCatcher demo script:
   Split one file into many PDFs, one page each.
"""

import sys
from reportlab.pdfgen import canvas
try:
    from rlextra.pageCatcher.pageCatcher import storeForms, restoreForms, fourPage
except ImportError:
    pass # running inside pageCatcher module?

def run():
    inputFileName = "sample2.pdf"
    StorageFile = "out7_split.data"
    page = 0
    done = 0
    while not done:
        try:
            print("from", inputFileName, "storing page", page, "to", StorageFile)
            storeForms(inputFileName, StorageFile, [page])
        except IndexError:
            print("stopping at page count", page)
            done = 1
        else:
            outputFileName = "out7_page"+str(page)+".pdf"
            print("restoring page into", outputFileName)
            canv = canvas.Canvas(outputFileName)
            names = restoreForms(StorageFile, canv)
            canv.doForm(names[0])
            canv.showPage()
            canv.save()
            print("   done with page", page)
            page = page+1

if __name__=='__main__':
    run()

    


__code__='Gas1]?ZX[c&Dsg)a"_9^-Dga8W)-[i:mjRWQqjPX`e9Wq`Q_d5N+b_VBJ53u/;_h9EAZW\'bk.`=UgKl=6a-G(IHmtE&JmKQmWQm"]>*7kl@/Idbl\'0Cr/kUj(E\'jm*l(Q"!!-C.gZ&K_!i1?Vi5P>4O:=Gn#+u:T"De)k!4DnE!BYgE!1*[M!*f^j$ePVF#M9,@&%D]9+lEO>NrVBKqulLf^s<!6!aYs+*:P_gS:+g"fDXSZ_QUmpK%C.YiXDUhZ3ePULmNV,k&V3-\'>"GC"fLW_%Td%KIp%ZRE5"T4/s%TK<=r2r7hph_*NoWk#6,^sV@qrI%#nDo8[D!0^]6!28AL@<$a0uc!(SjS#,.RXT0\\B]a/H;n%6/)fG-?\'(grs7/,,mUrCr>7kIkCCR*^(K,?G$6qZ#,^54&Q;eX)]AU/Xi)b&oF,7REKie\'M3"to,Y$>Q2Crg9f`/^X&r-)$GO(4X?*iZp2^uBB3Y=kNoI]9fh,arZGWu1AR&HbE?I<:-b?$+2k;KEK!t+q_-$P_g%##\\MNaFZ9>:FH\'#Z.FW^Cb@l^V*g_i8Fc1n,fM)>?Uq@bl>UZW(WtfWmEj-R1.aXh]gh:aW_@.oShKC+J,3o@qeVql\\g@3&kkV[2B3R:7.%^M_Kp(iN2DOnKcIYRTQPpj>;c/@>/E=pg5-jU`bkQguA/Q<GI]]Rf*-`K>OYehs7aX>^`ZgIAq849W+kd,Z3e>f6+R417.bEoc6l/E20p\'8/!"??VZZc^XE[^.\'.6Z_VH?$%L\'!G^#b_B<c??pZ[2N\'41FYKk*)N`MXiIK;C+V&$*??0\'OO:=4uoIu,huOPK%ueH$"i*2h/HGMHVq(IGEB8h[A$asZi1HdE^FsT%uQW2S`W1Q4,@@&*:unOWK5Ij^*c^7hFNA7($S*OBp?V;Mr\'8W035.)TL3>f2(RsGW8<>3!3@u^[3Smc%=06[YZ[r!$tlm.qi+IOkn^V.\'jjpEP[aL?<A%L)-+QZBBN?R(L$J6H@#GB6DO1aF+2fm"<QHNIQH_tDEF+O9;X>p"=V^RYC+7opTBPSG%aUKP6ufrK.#`$r<^a"Y03VFg&-~>'
